@@ -3,7 +3,7 @@
 A demo of K-Means clustering on the handwritten digits data
 ===========================================================
 
-In this example with compare the various initialization strategies for
+In this example we compare the various initialization strategies for
 K-means in terms of runtime and quality of the results.
 
 As the ground truth is known here, we also apply different cluster
@@ -29,7 +29,7 @@ print(__doc__)
 
 from time import time
 import numpy as np
-import pylab as pl
+import matplotlib.pyplot as plt
 
 from sklearn import metrics
 from sklearn.cluster import KMeans
@@ -105,23 +105,23 @@ Z = kmeans.predict(np.c_[xx.ravel(), yy.ravel()])
 
 # Put the result into a color plot
 Z = Z.reshape(xx.shape)
-pl.figure(1)
-pl.clf()
-pl.imshow(Z, interpolation='nearest',
-          extent=(xx.min(), xx.max(), yy.min(), yy.max()),
-          cmap=pl.cm.Paired,
-          aspect='auto', origin='lower')
+plt.figure(1)
+plt.clf()
+plt.imshow(Z, interpolation='nearest',
+           extent=(xx.min(), xx.max(), yy.min(), yy.max()),
+           cmap=plt.cm.Paired,
+           aspect='auto', origin='lower')
 
-pl.plot(reduced_data[:, 0], reduced_data[:, 1], 'k.', markersize=2)
+plt.plot(reduced_data[:, 0], reduced_data[:, 1], 'k.', markersize=2)
 # Plot the centroids as a white X
 centroids = kmeans.cluster_centers_
-pl.scatter(centroids[:, 0], centroids[:, 1],
-           marker='x', s=169, linewidths=3,
-           color='w', zorder=10)
-pl.title('K-means clustering on the digits dataset (PCA-reduced data)\n'
-         'Centroids are marked with white cross')
-pl.xlim(x_min, x_max)
-pl.ylim(y_min, y_max)
-pl.xticks(())
-pl.yticks(())
-pl.show()
+plt.scatter(centroids[:, 0], centroids[:, 1],
+            marker='x', s=169, linewidths=3,
+            color='w', zorder=10)
+plt.title('K-means clustering on the digits dataset (PCA-reduced data)\n'
+          'Centroids are marked with white cross')
+plt.xlim(x_min, x_max)
+plt.ylim(y_min, y_max)
+plt.xticks(())
+plt.yticks(())
+plt.show()
