@@ -10,6 +10,7 @@ from sklearn.utils.testing import assert_array_almost_equal
 from sklearn.utils.testing import SkipTest
 from sklearn.utils.testing import assert_almost_equal
 from sklearn.utils.testing import assert_raises
+from sklearn.utils.testing import assert_raises_regexp
 from sklearn.utils.testing import assert_true
 from sklearn.utils.testing import assert_greater
 from sklearn.utils.testing import assert_less
@@ -80,7 +81,7 @@ def test_labels_assignment_and_inertia():
 
 
 def test_minibatch_update_consistency():
-    """Check that dense and sparse minibatch update give the same results"""
+    # Check that dense and sparse minibatch update give the same results
     rng = np.random.RandomState(42)
     old_centers = centers + rng.normal(size=centers.shape)
 
@@ -163,15 +164,10 @@ def _check_fitted_model(km):
 
 def test_k_means_plus_plus_init():
     km = KMeans(init="k-means++", n_clusters=n_clusters,
-                     random_state=42).fit(X)
+                random_state=42).fit(X)
     _check_fitted_model(km)
 
 
-def test_k_means_check_fitted():
-    km = KMeans(n_clusters=n_clusters, random_state=42)
-    assert_raises(AttributeError, km._check_fitted)
-
-
 def test_k_means_new_centers():
     # Explore the part of the code where a new center is reassigned
     X = np.array([[0, 0, 1, 1],
@@ -181,9 +177,9 @@ def test_k_means_new_centers():
                   [0, 0, 0, 0],
                   [0, 1, 0, 0]])
     labels = [0, 1, 2, 1, 1, 2]
-    bad_centers = np.array([[+0,  1,  0,  0],
-                            [.2,  0, .2, .2],
-                            [+0,  0,  0,  0]])
+    bad_centers = np.array([[+0, 1, 0, 0],
+                            [.2, 0, .2, .2],
+                            [+0, 0, 0, 0]])
 
     km = KMeans(n_clusters=3, init=bad_centers, n_init=1, max_iter=10,
                 random_state=1)
@@ -207,10 +203,17 @@ def test_k_means_plus_plus_init_2_jobs():
         raise SkipTest('Multi-process bug with OpenBLAS (see issue #636)')
 
     km = KMeans(init="k-means++", n_clusters=n_clusters, n_jobs=2,
-                     random_state=42).fit(X)
+                random_state=42).fit(X)
     _check_fitted_model(km)
 
 
+def test_k_means_precompute_distances_flag():
+    # check that a warning is raised if the precompute_distances flag is not
+    # supported
+    km = KMeans(precompute_distances="wrong")
+    assert_raises(ValueError, km.fit, X)
+
+
 def test_k_means_plus_plus_init_sparse():
     km = KMeans(init="k-means++", n_clusters=n_clusters, random_state=42)
     km.fit(X_csr)
@@ -231,13 +234,13 @@ def test_k_means_random_init_sparse():
 
 def test_k_means_plus_plus_init_not_precomputed():
     km = KMeans(init="k-means++", n_clusters=n_clusters, random_state=42,
-                     precompute_distances=False).fit(X)
+                precompute_distances=False).fit(X)
     _check_fitted_model(km)
 
 
 def test_k_means_random_init_not_precomputed():
     km = KMeans(init="random", n_clusters=n_clusters, random_state=42,
-                     precompute_distances=False).fit(X)
+                precompute_distances=False).fit(X)
     _check_fitted_model(km)
 
 
@@ -248,6 +251,16 @@ def test_k_means_perfect_init():
     _check_fitted_model(km)
 
 
+def test_k_means_n_init():
+    rnd = np.random.RandomState(0)
+    X = rnd.normal(size=(40, 2))
+
+    # two regression tests on bad n_init argument
+    # previous bug: n_init <= 0 threw non-informative TypeError (#3858)
+    assert_raises_regexp(ValueError, "n_init", KMeans(n_init=0).fit, X)
+    assert_raises_regexp(ValueError, "n_init", KMeans(n_init=-1).fit, X)
+
+
 def test_mb_k_means_plus_plus_init_dense_array():
     mb_k_means = MiniBatchKMeans(init="k-means++", n_clusters=n_clusters,
                                  random_state=42)
@@ -320,14 +333,14 @@ def test_minibatch_sensible_reassign_fit():
                                        cluster_std=1., random_state=42)
     zeroed_X[::2, :] = 0
     mb_k_means = MiniBatchKMeans(n_clusters=20, batch_size=10, random_state=42,
-                                 verbose=10, init="random")
+                                 init="random")
     mb_k_means.fit(zeroed_X)
     # there should not be too many exact zero cluster centers
     assert_greater(mb_k_means.cluster_centers_.any(axis=1).sum(), 10)
 
     # do the same with batch-size > X.shape[0] (regression test)
     mb_k_means = MiniBatchKMeans(n_clusters=20, batch_size=201,
-                                 random_state=42, verbose=10, init="random")
+                                 random_state=42, init="random")
     mb_k_means.fit(zeroed_X)
     # there should not be too many exact zero cluster centers
     assert_greater(mb_k_means.cluster_centers_.any(axis=1).sum(), 10)
@@ -337,8 +350,7 @@ def test_minibatch_sensible_reassign_partial_fit():
     zeroed_X, true_labels = make_blobs(n_samples=n_samples, centers=5,
                                        cluster_std=1., random_state=42)
     zeroed_X[::2, :] = 0
-    mb_k_means = MiniBatchKMeans(n_clusters=20, random_state=42, verbose=10,
-                                 init="random")
+    mb_k_means = MiniBatchKMeans(n_clusters=20, random_state=42, init="random")
     for i in range(100):
         mb_k_means.partial_fit(zeroed_X)
     # there should not be too many exact zero cluster centers
@@ -468,7 +480,7 @@ def test_mini_match_k_means_invalid_init():
 
 
 def test_k_means_copyx():
-    """Check if copy_x=False returns nearly equal X after de-centering."""
+    # Check if copy_x=False returns nearly equal X after de-centering.
     my_X = X.copy()
     km = KMeans(copy_x=False, n_clusters=n_clusters, random_state=42)
     km.fit(my_X)
@@ -479,13 +491,11 @@ def test_k_means_copyx():
 
 
 def test_k_means_non_collapsed():
-    """Check k_means with a bad initialization does not yield a singleton
-
-    Starting with bad centers that are quickly ignored should not
-    result in a repositioning of the centers to the center of mass that
-    would lead to collapsed centers which in turns make the clustering
-    dependent of the numerical unstabilities.
-    """
+    # Check k_means with a bad initialization does not yield a singleton
+    # Starting with bad centers that are quickly ignored should not
+    # result in a repositioning of the centers to the center of mass that
+    # would lead to collapsed centers which in turns make the clustering
+    # dependent of the numerical unstabilities.
     my_X = np.array([[1.1, 1.1], [0.9, 1.1], [1.1, 0.9], [0.9, 1.1]])
     array_init = np.array([[1.0, 1.0], [5.0, 5.0], [-5.0, -5.0]])
     km = KMeans(init=array_init, n_clusters=3, random_state=42, n_init=1)
@@ -618,7 +628,7 @@ def test_fit_transform():
 
 
 def test_n_init():
-    """Check that increasing the number of init increases the quality"""
+    # Check that increasing the number of init increases the quality
     n_runs = 5
     n_init_range = [1, 5, 10]
     inertia = np.zeros((len(n_init_range), n_runs))
