@@ -5,12 +5,12 @@
 # License: BSD 3 clause
 
 import numpy as np
-import warnings
 
 from sklearn.utils.testing import assert_almost_equal
 from sklearn.utils.testing import assert_array_almost_equal
 from sklearn.utils.testing import assert_array_equal
 from sklearn.utils.testing import assert_raises
+from sklearn.utils.testing import assert_warns
 
 from sklearn import datasets
 from sklearn.covariance import empirical_covariance, EmpiricalCovariance, \
@@ -23,9 +23,7 @@ n_samples, n_features = X.shape
 
 
 def test_covariance():
-    """Tests Covariance module on a simple dataset.
-
-    """
+    # Tests Covariance module on a simple dataset.
     # test covariance fit from data
     cov = EmpiricalCovariance()
     cov.fit(X)
@@ -57,10 +55,12 @@ def test_covariance():
         cov.error_norm(empirical_covariance(X_1d), norm='spectral'), 0)
 
     # test with one sample
+    # FIXME I don't know what this test does
     X_1sample = np.arange(5)
     cov = EmpiricalCovariance()
-    with warnings.catch_warnings(record=True):
-        cov.fit(X_1sample)
+    assert_warns(UserWarning, cov.fit, X_1sample)
+    assert_array_almost_equal(cov.covariance_,
+                              np.zeros(shape=(5, 5), dtype=np.float64))
 
     # test integer type
     X_integer = np.asarray([[0, 1], [1, 0]])
@@ -74,9 +74,7 @@ def test_covariance():
 
 
 def test_shrunk_covariance():
-    """Tests ShrunkCovariance module on a simple dataset.
-
-    """
+    # Tests ShrunkCovariance module on a simple dataset.
     # compare shrunk covariance obtained from data and from MLE estimate
     cov = ShrunkCovariance(shrinkage=0.5)
     cov.fit(X)
@@ -108,9 +106,7 @@ def test_shrunk_covariance():
 
 
 def test_ledoit_wolf():
-    """Tests LedoitWolf module on a simple dataset.
-
-    """
+    # Tests LedoitWolf module on a simple dataset.
     # test shrinkage coeff on a simple data set
     X_centered = X - X.mean(axis=0)
     lw = LedoitWolf(assume_centered=True)
@@ -180,10 +176,12 @@ def test_ledoit_wolf():
     assert_array_almost_equal(empirical_covariance(X_1d), lw.covariance_, 4)
 
     # test with one sample
+    # FIXME I don't know what this test does
     X_1sample = np.arange(5)
     lw = LedoitWolf()
-    with warnings.catch_warnings(record=True):
-        lw.fit(X_1sample)
+    assert_warns(UserWarning, lw.fit, X_1sample)
+    assert_array_almost_equal(lw.covariance_,
+                              np.zeros(shape=(5, 5), dtype=np.float64))
 
     # test shrinkage coeff on a simple data set (without saving precision)
     lw = LedoitWolf(store_precision=False)
@@ -193,9 +191,7 @@ def test_ledoit_wolf():
 
 
 def test_oas():
-    """Tests OAS module on a simple dataset.
-
-    """
+    # Tests OAS module on a simple dataset.
     # test shrinkage coeff on a simple data set
     X_centered = X - X.mean(axis=0)
     oa = OAS(assume_centered=True)
@@ -227,7 +223,7 @@ def test_oas():
     assert_almost_equal(oa.score(X_centered), score_, 4)
     assert(oa.precision_ is None)
 
-    ### Same tests without assuming centered data
+    # Same tests without assuming centered data--------------------------------
     # test shrinkage coeff on a simple data set
     oa = OAS()
     oa.fit(X)
@@ -252,10 +248,12 @@ def test_oas():
     assert_array_almost_equal(empirical_covariance(X_1d), oa.covariance_, 4)
 
     # test with one sample
+    # FIXME I don't know what this test does
     X_1sample = np.arange(5)
     oa = OAS()
-    with warnings.catch_warnings(record=True):
-        oa.fit(X_1sample)
+    assert_warns(UserWarning, oa.fit, X_1sample)
+    assert_array_almost_equal(oa.covariance_,
+                              np.zeros(shape=(5, 5), dtype=np.float64))
 
     # test shrinkage coeff on a simple data set (without saving precision)
     oa = OAS(store_precision=False)
