@@ -15,6 +15,8 @@ from .univariate_selection import SelectFdr
 from .univariate_selection import SelectFwe
 from .univariate_selection import GenericUnivariateSelect
 
+from .variance_threshold import VarianceThreshold
+
 from .rfe import RFE
 from .rfe import RFECV
 
@@ -26,6 +28,7 @@ __all__ = ['GenericUnivariateSelect',
            'SelectFwe',
            'SelectKBest',
            'SelectPercentile',
+           'VarianceThreshold',
            'chi2',
            'f_classif',
            'f_oneway',
