@@ -9,10 +9,9 @@ from sklearn.utils.testing import assert_less
 from sklearn.utils.testing import assert_greater
 from sklearn.utils.testing import assert_raises
 from sklearn.utils.testing import ignore_warnings, assert_warns_message
-from sklearn.utils.testing import assert_warns
+from sklearn.utils.testing import assert_no_warnings, assert_warns
 from sklearn.utils import ConvergenceWarning
 from sklearn import linear_model, datasets
-from nose import SkipTest
 
 diabetes = datasets.load_diabetes()
 X, y = diabetes.data, diabetes.target
@@ -21,9 +20,7 @@ X, y = diabetes.data, diabetes.target
 
 
 def test_simple():
-    """
-    Principle of Lars is to keep covariances tied and decreasing
-    """
+    # Principle of Lars is to keep covariances tied and decreasing
 
     # also test verbose output
     from sklearn.externals.six.moves import cStringIO as StringIO
@@ -53,9 +50,7 @@ def test_simple():
 
 
 def test_simple_precomputed():
-    """
-    The same, with precomputed Gram matrix
-    """
+    # The same, with precomputed Gram matrix
 
     G = np.dot(diabetes.data.T, diabetes.data)
     alphas_, active, coef_path_ = linear_model.lars_path(
@@ -75,9 +70,7 @@ def test_simple_precomputed():
 
 
 def test_all_precomputed():
-    """
-    Test that lars_path with precomputed Gram and Xy gives the right answer
-    """
+    # Test that lars_path with precomputed Gram and Xy gives the right answer
     X, y = diabetes.data, diabetes.target
     G = np.dot(X.T, X)
     Xy = np.dot(X.T, y)
@@ -89,10 +82,8 @@ def test_all_precomputed():
 
 
 def test_lars_lstsq():
-    """
-    Test that Lars gives least square solution at the end
-    of the path
-    """
+    # Test that Lars gives least square solution at the end
+    # of the path
     X1 = 3 * diabetes.data  # use un-normalized dataset
     clf = linear_model.LassoLars(alpha=0.)
     clf.fit(X1, y)
@@ -101,17 +92,15 @@ def test_lars_lstsq():
 
 
 def test_lasso_gives_lstsq_solution():
-    """
-    Test that Lars Lasso gives least square solution at the end
-    of the path
-    """
+    # Test that Lars Lasso gives least square solution at the end
+    # of the path
     alphas_, active, coef_path_ = linear_model.lars_path(X, y, method="lasso")
     coef_lstsq = np.linalg.lstsq(X, y)[0]
     assert_array_almost_equal(coef_lstsq, coef_path_[:, -1])
 
 
 def test_collinearity():
-    """Check that lars_path is robust to collinearity in input"""
+    # Check that lars_path is robust to collinearity in input
     X = np.array([[3., 3., 1.],
                   [2., 2., 0.],
                   [1., 1., 0]])
@@ -134,9 +123,7 @@ def test_collinearity():
 
 
 def test_no_path():
-    """
-    Test that the ``return_path=False`` option returns the correct output
-    """
+    # Test that the ``return_path=False`` option returns the correct output
 
     alphas_, active_, coef_path_ = linear_model.lars_path(
         diabetes.data, diabetes.target, method="lar")
@@ -148,9 +135,7 @@ def test_no_path():
 
 
 def test_no_path_precomputed():
-    """
-    Test that the ``return_path=False`` option with Gram remains correct
-    """
+    # Test that the ``return_path=False`` option with Gram remains correct
 
     G = np.dot(diabetes.data.T, diabetes.data)
 
@@ -165,9 +150,7 @@ def test_no_path_precomputed():
 
 
 def test_no_path_all_precomputed():
-    """
-    Test that the ``return_path=False`` option with Gram and Xy remains correct
-    """
+    # Test that the ``return_path=False`` option with Gram and Xy remains correct
     X, y = 3 * diabetes.data, diabetes.target
     G = np.dot(X.T, X)
     Xy = np.dot(X.T, y)
@@ -184,14 +167,9 @@ def test_no_path_all_precomputed():
 
 def test_singular_matrix():
     # Test when input is a singular matrix
-    # In this test the "drop for good strategy" of lars_path is necessary
-    # to give a good answer
     X1 = np.array([[1, 1.], [1., 1.]])
     y1 = np.array([1, 1])
-    in_warn_message = 'Dropping a regressor'
-    f = assert_warns_message
-    alphas, active, coef_path = f(ConvergenceWarning, in_warn_message,
-                                  linear_model.lars_path, X1, y1)
+    alphas, active, coef_path = linear_model.lars_path(X1, y1)
     assert_array_almost_equal(coef_path.T, [[0, 0], [1, 0]])
 
 
@@ -223,10 +201,8 @@ def test_rank_deficient_design():
 
 
 def test_lasso_lars_vs_lasso_cd(verbose=False):
-    """
-    Test that LassoLars and Lasso using coordinate descent give the
-    same results.
-    """
+    # Test that LassoLars and Lasso using coordinate descent give the
+    # same results.
     X = 3 * diabetes.data
 
     alphas, _, lasso_path = linear_model.lars_path(X, y, method='lasso')
@@ -262,11 +238,9 @@ def test_lasso_lars_vs_lasso_cd(verbose=False):
 
 
 def test_lasso_lars_vs_lasso_cd_early_stopping(verbose=False):
-    """
-    Test that LassoLars and Lasso using coordinate descent give the
-    same results when early stopping is used.
-    (test : before, in the middle, and in the last part of the path)
-    """
+    # Test that LassoLars and Lasso using coordinate descent give the
+    # same results when early stopping is used.
+    # (test : before, in the middle, and in the last part of the path)
     alphas_min = [10, 0.9, 1e-4]
     for alphas_min in alphas_min:
         alphas, _, lasso_path = linear_model.lars_path(X, y, method='lasso',
@@ -291,10 +265,6 @@ def test_lasso_lars_vs_lasso_cd_early_stopping(verbose=False):
 
 
 def test_lasso_lars_path_length():
-    import struct
-    if struct.calcsize('P') * 8 == 32:
-        # FIXME in master
-        raise SkipTest("skip unstable test on 32 bit arch")
     # Test that the path length of the LassoLars is right
     lasso = linear_model.LassoLars()
     lasso.fit(X, y)
@@ -327,39 +297,24 @@ def test_lasso_lars_vs_lasso_cd_ill_conditioned():
     sigma = 0.2
     y += sigma * rng.rand(*y.shape)
     y = y.squeeze()
-
-    f = assert_warns_message
-
-    def in_warn_message(msg):
-        return 'Early stopping' in msg or 'Dropping a regressor' in msg
-    lars_alphas, _, lars_coef = f(ConvergenceWarning,
-                                  in_warn_message,
-                                  linear_model.lars_path, X, y, method='lasso')
-
-    with ignore_warnings():
-        _, lasso_coef2, _ = linear_model.lasso_path(X, y,
-                                                    alphas=lars_alphas,
-                                                    tol=1e-6,
-                                                    fit_intercept=False)
-
-        lasso_coef = np.zeros((w.shape[0], len(lars_alphas)))
-        iter_models = enumerate(linear_model.lasso_path(X, y,
-                                                        alphas=lars_alphas,
-                                                        tol=1e-6,
-                                                        return_models=True,
-                                                        fit_intercept=False))
-        for i, model in iter_models:
-            lasso_coef[:, i] = model.coef_
-
-    np.testing.assert_array_almost_equal(lars_coef, lasso_coef, decimal=1)
-    np.testing.assert_array_almost_equal(lars_coef, lasso_coef2, decimal=1)
-    np.testing.assert_array_almost_equal(lasso_coef, lasso_coef2, decimal=1)
+    lars_alphas, _, lars_coef = linear_model.lars_path(X, y, method='lasso')
 
 
-def test_lars_drop_for_good():
+    _, lasso_coef2, _ = linear_model.lasso_path(X, y,
+                                                alphas=lars_alphas,
+                                                tol=1e-6,
+                                                fit_intercept=False)
+
+    assert_array_almost_equal(lars_coef, lasso_coef2, decimal=1)
+
+
+def test_lasso_lars_vs_lasso_cd_ill_conditioned2():
     # Create an ill-conditioned situation in which the LARS has to go
     # far in the path to converge, and check that LARS and coordinate
     # descent give the same answers
+    # Note it used to be the case that Lars had to use the drop for good
+    # strategy for this but this is no longer the case with the
+    # equality_tolerance checks
     X = [[1e20,  1e20,  0],
          [-1e-32,  0,  0],
          [1,       1,  1]]
@@ -376,19 +331,15 @@ def test_lars_drop_for_good():
     lars_obj = objective_function(lars_coef_)
 
     coord_descent = linear_model.Lasso(alpha=alpha, tol=1e-10, normalize=False)
-    with ignore_warnings():
-        cd_coef_ = coord_descent.fit(X, y).coef_
+    cd_coef_ = coord_descent.fit(X, y).coef_
     cd_obj = objective_function(cd_coef_)
 
     assert_less(lars_obj, cd_obj * (1. + 1e-8))
 
 
 def test_lars_add_features():
-    """
-    assure that at least some features get added if necessary
-
-    test for 6d2b4c
-    """
+    # assure that at least some features get added if necessary
+    # test for 6d2b4c
     # Hilbert matrix
     n = 5
     H = 1. / (np.arange(1, n + 1) + np.arange(n)[:, np.newaxis])
@@ -407,9 +358,7 @@ def test_lars_n_nonzero_coefs(verbose=False):
 
 
 def test_multitarget():
-    """
-    Assure that estimators receiving multidimensional y do the right thing
-    """
+    # Assure that estimators receiving multidimensional y do the right thing
     X = diabetes.data
     Y = np.vstack([diabetes.target, diabetes.target ** 2]).T
     n_targets = Y.shape[1]
@@ -432,12 +381,10 @@ def test_multitarget():
 
 
 def test_lars_cv():
-    """ Test the LassoLarsCV object by checking that the optimal alpha
-        increases as the number of samples increases.
-
-        This property is not actually garantied in general and is just a
-        property of the given dataset, with the given steps chosen.
-    """
+    # Test the LassoLarsCV object by checking that the optimal alpha
+    # increases as the number of samples increases.
+    # This property is not actually garantied in general and is just a
+    # property of the given dataset, with the given steps chosen.
     old_alpha = 0
     lars_cv = linear_model.LassoLarsCV()
     for length in (400, 200, 100):
@@ -449,11 +396,10 @@ def test_lars_cv():
 
 
 def test_lasso_lars_ic():
-    """ Test the LassoLarsIC object by checking that
-        - some good features are selected.
-        - alpha_bic > alpha_aic
-        - n_nonzero_bic < n_nonzero_aic
-    """
+    # Test the LassoLarsIC object by checking that
+    # - some good features are selected.
+    # - alpha_bic > alpha_aic
+    # - n_nonzero_bic < n_nonzero_aic
     lars_bic = linear_model.LassoLarsIC('bic')
     lars_aic = linear_model.LassoLarsIC('aic')
     rng = np.random.RandomState(42)
@@ -473,6 +419,15 @@ def test_lasso_lars_ic():
     assert_raises(ValueError, lars_broken.fit, X, y)
 
 
+def test_no_warning_for_zero_mse():
+    # LassoLarsIC should not warn for log of zero MSE.
+    y = np.arange(10, dtype=float)
+    X = y.reshape(-1, 1)
+    lars = linear_model.LassoLarsIC(normalize=False)
+    assert_no_warnings(lars.fit, X, y)
+    assert_true(np.any(np.isinf(lars.criterion_)))
+
+
 if __name__ == '__main__':
     import nose
     nose.runmodule()
