@@ -15,13 +15,14 @@ import scipy.sparse as sp
 from ..base import BaseEstimator
 from ..base import TransformerMixin
 from ..externals.six.moves import xrange
-from ..utils import atleast2d_or_csr, check_arrays
+from ..utils import check_array
 from ..utils import check_random_state
 from ..utils import gen_even_slices
 from ..utils import issparse
 from ..utils.extmath import safe_sparse_dot
 from ..utils.extmath import log_logistic
 from ..utils.fixes import expit             # logistic function
+from ..utils.validation import check_is_fitted
 
 
 class BernoulliRBM(BaseEstimator, TransformerMixin):
@@ -62,13 +63,13 @@ class BernoulliRBM(BaseEstimator, TransformerMixin):
 
     Attributes
     ----------
-    `intercept_hidden_` : array-like, shape (n_components,)
+    intercept_hidden_ : array-like, shape (n_components,)
         Biases of the hidden units.
 
-    `intercept_visible_` : array-like, shape (n_features,)
+    intercept_visible_ : array-like, shape (n_features,)
         Biases of the visible units.
 
-    `components_` : array-like, shape (n_components, n_features)
+    components_ : array-like, shape (n_components, n_features)
         Weight matrix, where n_features in the number of
         visible units and n_components is the number of hidden units.
 
@@ -116,7 +117,9 @@ class BernoulliRBM(BaseEstimator, TransformerMixin):
         h : array, shape (n_samples, n_components)
             Latent representations of the data.
         """
-        X, = check_arrays(X, sparse_format='csr', dtype=np.float)
+        check_is_fitted(self, "components_")
+
+        X = check_array(X, accept_sparse='csr', dtype=np.float)
         return self._mean_hiddens(X)
 
     def _mean_hiddens(self, v):
@@ -206,13 +209,15 @@ class BernoulliRBM(BaseEstimator, TransformerMixin):
         v_new : array-like, shape (n_samples, n_features)
             Values of the visible layer after one Gibbs step.
         """
-        rng = check_random_state(self.random_state)
-        h_ = self._sample_hiddens(v, rng)
-        v_ = self._sample_visibles(h_, rng)
+        check_is_fitted(self, "components_")
+        if not hasattr(self, "random_state_"):
+            self.random_state_ = check_random_state(self.random_state)
+        h_ = self._sample_hiddens(v, self.random_state_)
+        v_ = self._sample_visibles(h_, self.random_state_)
 
         return v_
 
-    def partial_fit(self, X):
+    def partial_fit(self, X, y=None):
         """Fit the model to the data X which should contain a partial
         segment of the data.
 
@@ -226,7 +231,7 @@ class BernoulliRBM(BaseEstimator, TransformerMixin):
         self : BernoulliRBM
             The fitted model.
         """
-        X, = check_arrays(X, sparse_format='csr', dtype=np.float)
+        X = check_array(X, accept_sparse='csr', dtype=np.float)
         if not hasattr(self, 'random_state_'):
             self.random_state_ = check_random_state(self.random_state)
         if not hasattr(self, 'components_'):
@@ -295,7 +300,9 @@ class BernoulliRBM(BaseEstimator, TransformerMixin):
         free energy on X, then on a randomly corrupted version of X, and
         returns the log of the logistic function of the difference.
         """
-        v = atleast2d_or_csr(X)
+        check_is_fitted(self, "components_")
+
+        v = check_array(X, accept_sparse='csr')
         rng = check_random_state(self.random_state)
 
         # Randomly corrupt one feature in each sample in v.
@@ -325,7 +332,7 @@ class BernoulliRBM(BaseEstimator, TransformerMixin):
         self : BernoulliRBM
             The fitted model.
         """
-        X, = check_arrays(X, sparse_format='csr', dtype=np.float)
+        X = check_array(X, accept_sparse='csr', dtype=np.float)
         n_samples = X.shape[0]
         rng = check_random_state(self.random_state)
 
