@@ -21,7 +21,8 @@ from ..utils.fixes import sparse_min_max
 from ..utils.fixes import astype
 from ..utils.fixes import in1d
 from ..utils import deprecated, column_or_1d
-from ..utils.validation import check_arrays
+from ..utils.validation import check_array
+from ..utils.validation import _num_samples
 from ..utils.multiclass import unique_labels
 from ..utils.multiclass import type_of_target
 
@@ -34,6 +35,7 @@ __all__ = [
     'label_binarize',
     'LabelBinarizer',
     'LabelEncoder',
+    'MultiLabelBinarizer',
 ]
 
 
@@ -56,7 +58,7 @@ class LabelEncoder(BaseEstimator, TransformerMixin):
 
     Attributes
     ----------
-    `classes_` : array of shape (n_class,)
+    classes_ : array of shape (n_class,)
         Holds the label for each class.
 
     Examples
@@ -199,21 +201,28 @@ class LabelBinarizer(BaseEstimator, TransformerMixin):
 
     Attributes
     ----------
-    `classes_` : array of shape [n_class]
+    classes_ : array of shape [n_class]
         Holds the label for each class.
 
-    `multilabel_` : boolean
+    y_type_ : str,
+        Represents the type of the target data as evaluated by
+        utils.multiclass.type_of_target. Possible type are 'continuous',
+        'continuous-multioutput', 'binary', 'multiclass',
+        'mutliclass-multioutput', 'multilabel-sequences',
+        'multilabel-indicator', and 'unknown'.
+
+    multilabel_ : boolean
         True if the transformer was fitted on a multilabel rather than a
-        multiclass set of labels. The multilabel_ attribute is deprecated
+        multiclass set of labels. The ``multilabel_`` attribute is deprecated
         and will be removed in 0.18
 
-    `sparse_input_` : boolean,
+    sparse_input_ : boolean,
         True if the input data to transform is given as a sparse matrix, False
         otherwise.
 
-    `indicator_matrix_` : str
+    indicator_matrix_ : str
         'sparse' when the input data to tansform is a multilable-indicator and
-        is sparse, None otherwise. The indicator_matrix_ attribute is
+        is sparse, None otherwise. The ``indicator_matrix_`` attribute is
         deprecated as of version 0.16 and will be removed in 0.18
 
 
@@ -230,6 +239,7 @@ class LabelBinarizer(BaseEstimator, TransformerMixin):
            [0, 0, 0, 1]])
 
     Binary targets transform to a column vector
+
     >>> lb = preprocessing.LabelBinarizer()
     >>> lb.fit_transform(['yes', 'no', 'no', 'yes'])
     array([[1],
@@ -237,11 +247,18 @@ class LabelBinarizer(BaseEstimator, TransformerMixin):
            [0],
            [1]])
 
+    Passing a 2D matrix for multilabel classification
+
     >>> import numpy as np
     >>> lb.fit(np.array([[0, 1, 1], [1, 0, 0]]))
     LabelBinarizer(neg_label=0, pos_label=1, sparse_output=False)
     >>> lb.classes_
     array([0, 1, 2])
+    >>> lb.transform([0, 1, 2, 1])
+    array([[1, 0, 0],
+           [0, 1, 0],
+           [0, 0, 1],
+           [0, 1, 0]])
 
     See also
     --------
@@ -265,21 +282,15 @@ class LabelBinarizer(BaseEstimator, TransformerMixin):
         self.sparse_output = sparse_output
 
     @property
-    @deprecated("Attribute `multilabel` was renamed to `multilabel_` in "
-                "0.14 and will be removed in 0.16")
-    def multilabel(self):
-        return self.multilabel_
-
-    @property
-    @deprecated("Attribute indicator_matrix_ is deprecated and will be "
-                "removed in 0.17. Use 'y_type_ == 'multilabel-indicator'' "
+    @deprecated("Attribute ``indicator_matrix_`` is deprecated and will be "
+                "removed in 0.17. Use ``y_type_ == 'multilabel-indicator'`` "
                 "instead")
     def indicator_matrix_(self):
         return self.y_type_ == 'multilabel-indicator'
 
     @property
-    @deprecated("Attribute multilabel_ is deprecated and will be removed "
-                "in 0.17. Use 'y_type_.startswith('multilabel')' "
+    @deprecated("Attribute ``multilabel_`` is deprecated and will be removed "
+                "in 0.17. Use ``y_type_.startswith('multilabel')`` "
                 "instead")
     def multilabel_(self):
         return self.y_type_.startswith('multilabel')
@@ -302,6 +313,12 @@ class LabelBinarizer(BaseEstimator, TransformerMixin):
         self : returns an instance of self.
         """
         self.y_type_ = type_of_target(y)
+        if 'multioutput' in self.y_type_:
+            raise ValueError("Multioutput target data is not supported with "
+                             "label binarization")
+        if _num_samples(y) == 0:
+            raise ValueError('y has 0 samples: %r' % y)
+
         self.sparse_input_ = sp.issparse(y)
         self.classes_ = unique_labels(y)
         return self
@@ -450,8 +467,10 @@ def label_binarize(y, classes, neg_label=0, pos_label=1,
     if not isinstance(y, list):
         # XXX Workaround that will be removed when list of list format is
         # dropped
-        y, = check_arrays(y)
-
+        y = check_array(y, accept_sparse='csr', ensure_2d=False, dtype=None)
+    else:
+        if _num_samples(y) == 0:
+            raise ValueError('y has 0 samples: %r' % y)
     if neg_label >= pos_label:
         raise ValueError("neg_label={0} must be strictly less than "
                          "pos_label={1}.".format(neg_label, pos_label))
@@ -474,6 +493,9 @@ def label_binarize(y, classes, neg_label=0, pos_label=1,
         pos_label = -neg_label
 
     y_type = type_of_target(y)
+    if 'multioutput' in y_type:
+        raise ValueError("Multioutput target data is not supported with label "
+                         "binarization")
 
     n_samples = y.shape[0] if sp.issparse(y) else len(y)
     n_classes = len(classes)
@@ -532,6 +554,8 @@ def label_binarize(y, classes, neg_label=0, pos_label=1,
 
         if pos_switch:
             Y[Y == pos_label] = 0
+    else:
+        Y.data = astype(Y.data, int, copy=False)
 
     # preserve label ordering
     if np.any(classes != sorted_class):
@@ -539,7 +563,10 @@ def label_binarize(y, classes, neg_label=0, pos_label=1,
         Y = Y[:, indices]
 
     if y_type == "binary":
-        Y = Y[:, -1].reshape((-1, 1))
+        if sparse_output:
+            Y = Y.getcol(-1)
+        else:
+            Y = Y[:, -1].reshape((-1, 1))
 
     return Y
 
@@ -580,7 +607,7 @@ def _inverse_binarize_multiclass(y, classes):
         samples = np.arange(n_samples)[(row_nnz > 0) &
                                        (row_max.ravel() == 0)]
         for i in samples:
-            ind = y.indices[y.indptr[i]:y.indptr[i+1]]
+            ind = y.indices[y.indptr[i]:y.indptr[i + 1]]
             y_i_argmax[i] = classes[np.setdiff1d(outputs, ind)][0]
 
         return classes[y_i_argmax]
@@ -615,6 +642,8 @@ def _inverse_binarize_thresholding(y, output_type, classes, threshold):
 
     # Inverse transform data
     if output_type == "binary":
+        if sp.issparse(y):
+            y = y.toarray()
         if y.ndim == 2 and y.shape[1] == 2:
             return classes[y[:, 1]]
         else:
@@ -654,9 +683,12 @@ class MultiLabelBinarizer(BaseEstimator, TransformerMixin):
     classes : array-like of shape [n_classes] (optional)
         Indicates an ordering for the class labels
 
+    sparse_output : boolean (default: False),
+        Set to true if output binary array is desired in CSR sparse format
+
     Attributes
     ----------
-    `classes_` : array of labels
+    classes_ : array of labels
         A copy of the `classes` parameter where provided,
         or otherwise, the sorted set of classes found when fitting.
 
