@@ -9,20 +9,24 @@ from sklearn.externals.six.moves import xrange
 from itertools import chain, product
 import pickle
 import sys
-import warnings
 
 import numpy as np
 import scipy.sparse as sp
 
 from sklearn.utils.testing import assert_equal
+from sklearn.utils.testing import assert_not_equal
 from sklearn.utils.testing import assert_raises
+from sklearn.utils.testing import assert_warns
 from sklearn.utils.testing import assert_raise_message
 from sklearn.utils.testing import assert_false, assert_true
 from sklearn.utils.testing import assert_array_equal
 from sklearn.utils.testing import assert_almost_equal
 from sklearn.utils.testing import assert_array_almost_equal
+from sklearn.utils.testing import assert_no_warnings
+from sklearn.utils.testing import ignore_warnings
+from sklearn.utils.mocking import CheckingClassifier, MockDataFrame
 
-from scipy.stats import distributions
+from scipy.stats import bernoulli, expon, uniform
 
 from sklearn.externals.six.moves import zip
 from sklearn.base import BaseEstimator
@@ -30,15 +34,17 @@ from sklearn.datasets import make_classification
 from sklearn.datasets import make_blobs
 from sklearn.datasets import make_multilabel_classification
 from sklearn.grid_search import (GridSearchCV, RandomizedSearchCV,
-                                 ParameterGrid, ParameterSampler)
+                                 ParameterGrid, ParameterSampler,
+                                 ChangedBehaviorWarning)
 from sklearn.svm import LinearSVC, SVC
 from sklearn.tree import DecisionTreeRegressor
 from sklearn.tree import DecisionTreeClassifier
-from sklearn.cluster import KMeans, SpectralClustering
+from sklearn.cluster import KMeans
+from sklearn.neighbors import KernelDensity
 from sklearn.metrics import f1_score
 from sklearn.metrics import make_scorer
 from sklearn.metrics import roc_auc_score
-from sklearn.cross_validation import KFold, StratifiedKFold
+from sklearn.cross_validation import KFold, StratifiedKFold, FitFailedWarning
 from sklearn.preprocessing import Imputer
 from sklearn.pipeline import Pipeline
 
@@ -76,37 +82,6 @@ class MockClassifier(object):
         return self
 
 
-class MockListClassifier(object):
-    """Dummy classifier to test the cross-validation.
-
-    Checks that GridSearchCV didn't convert X to array.
-    """
-    def __init__(self, foo_param=0):
-        self.foo_param = foo_param
-
-    def fit(self, X, Y):
-        assert_true(len(X) == len(Y))
-        assert_true(isinstance(X, list))
-        return self
-
-    def predict(self, T):
-        return T.shape[0]
-
-    def score(self, X=None, Y=None):
-        if self.foo_param > 1:
-            score = 1.
-        else:
-            score = 0.
-        return score
-
-    def get_params(self, deep=False):
-        return {'foo_param': self.foo_param}
-
-    def set_params(self, **params):
-        self.foo_param = params['foo_param']
-        return self
-
-
 class LinearSVCNoScore(LinearSVC):
     """An LinearSVC classifier that has no score method."""
     @property
@@ -118,7 +93,7 @@ y = np.array([1, 1, 2, 2])
 
 
 def test_parameter_grid():
-    """Test basic properties of ParameterGrid."""
+    # Test basic properties of ParameterGrid.
     params1 = {"foo": [1, 2, 3]}
     grid1 = ParameterGrid(params1)
     assert_true(isinstance(grid1, Iterable))
@@ -149,7 +124,7 @@ def test_parameter_grid():
 
 
 def test_grid_search():
-    """Test that the best estimator contains the right value for foo_param"""
+    # Test that the best estimator contains the right value for foo_param
     clf = MockClassifier()
     grid_search = GridSearchCV(clf, {'foo_param': [1, 2, 3]}, verbose=3)
     # make sure it selects the smallest parameter in case of ties
@@ -173,13 +148,14 @@ def test_grid_search():
     assert_raises(ValueError, grid_search.fit, X, y)
 
 
+@ignore_warnings
 def test_grid_search_no_score():
     # Test grid-search on classifier that has no score function.
     clf = LinearSVC(random_state=0)
     X, y = make_blobs(random_state=0, centers=2)
     Cs = [.1, 1, 10]
     clf_no_score = LinearSVCNoScore(random_state=0)
-    grid_search = GridSearchCV(clf, {'C': Cs})
+    grid_search = GridSearchCV(clf, {'C': Cs}, scoring='accuracy')
     grid_search.fit(X, y)
 
     grid_search_no_score = GridSearchCV(clf_no_score, {'C': Cs},
@@ -198,23 +174,51 @@ def test_grid_search_no_score():
                          [[1]])
 
 
-def test_trivial_grid_scores():
-    """Test search over a "grid" with only one point.
+def test_grid_search_score_method():
+    X, y = make_classification(n_samples=100, n_classes=2, flip_y=.2,
+                               random_state=0)
+    clf = LinearSVC(random_state=0)
+    grid = {'C': [.1]}
 
-    Non-regression test: grid_scores_ wouldn't be set by GridSearchCV.
-    """
+    search_no_scoring = GridSearchCV(clf, grid, scoring=None).fit(X, y)
+    search_accuracy = GridSearchCV(clf, grid, scoring='accuracy').fit(X, y)
+    search_no_score_method_auc = GridSearchCV(LinearSVCNoScore(), grid,
+                                              scoring='roc_auc').fit(X, y)
+    search_auc = GridSearchCV(clf, grid, scoring='roc_auc').fit(X, y)
+
+    # Check warning only occurs in situation where behavior changed:
+    # estimator requires score method to compete with scoring parameter
+    score_no_scoring = assert_no_warnings(search_no_scoring.score, X, y)
+    score_accuracy = assert_warns(ChangedBehaviorWarning,
+                                  search_accuracy.score, X, y)
+    score_no_score_auc = assert_no_warnings(search_no_score_method_auc.score,
+                                            X, y)
+    score_auc = assert_warns(ChangedBehaviorWarning,
+                             search_auc.score, X, y)
+    # ensure the test is sane
+    assert_true(score_auc < 1.0)
+    assert_true(score_accuracy < 1.0)
+    assert_not_equal(score_auc, score_accuracy)
+
+    assert_almost_equal(score_accuracy, score_no_scoring)
+    assert_almost_equal(score_auc, score_no_score_auc)
+
+
+def test_trivial_grid_scores():
+    # Test search over a "grid" with only one point.
+    # Non-regression test: grid_scores_ wouldn't be set by GridSearchCV.
     clf = MockClassifier()
     grid_search = GridSearchCV(clf, {'foo_param': [1]})
     grid_search.fit(X, y)
     assert_true(hasattr(grid_search, "grid_scores_"))
 
-    random_search = RandomizedSearchCV(clf, {'foo_param': [0]})
+    random_search = RandomizedSearchCV(clf, {'foo_param': [0]}, n_iter=1)
     random_search.fit(X, y)
     assert_true(hasattr(random_search, "grid_scores_"))
 
 
 def test_no_refit():
-    """Test that grid search can be used for model selection only"""
+    # Test that grid search can be used for model selection only
     clf = MockClassifier()
     grid_search = GridSearchCV(clf, {'foo_param': [1, 2, 3]}, refit=False)
     grid_search.fit(X, y)
@@ -222,8 +226,8 @@ def test_no_refit():
 
 
 def test_grid_search_error():
-    """Test that grid search will capture errors on data with different
-    length"""
+    # Test that grid search will capture errors on data with different
+    # length
     X_, y_ = make_classification(n_samples=200, n_features=100, random_state=0)
 
     clf = LinearSVC()
@@ -299,7 +303,7 @@ def test_grid_search_bad_param_grid():
 
 
 def test_grid_search_sparse():
-    """Test that grid search works with both dense and sparse matrices"""
+    # Test that grid search works with both dense and sparse matrices
     X_, y_ = make_classification(n_samples=200, n_features=100, random_state=0)
 
     clf = LinearSVC()
@@ -338,8 +342,8 @@ def test_grid_search_sparse_scoring():
     assert_array_equal(y_pred, y_pred2)
     assert_equal(C, C2)
     # Smoke test the score
-    #np.testing.assert_allclose(f1_score(cv.predict(X_[:180]), y[:180]),
-    #                        cv.score(X_[:180], y[:180]))
+    # np.testing.assert_allclose(f1_score(cv.predict(X_[:180]), y[:180]),
+    #                            cv.score(X_[:180], y[:180]))
 
     # test loss where greater is worse
     def f1_loss(y_true_, y_pred_):
@@ -354,46 +358,9 @@ def test_grid_search_sparse_scoring():
     assert_array_equal(y_pred, y_pred3)
 
 
-def test_deprecated_score_func():
-    # test that old deprecated way of passing a score / loss function is still
-    # supported
-    X, y = make_classification(n_samples=200, n_features=100, random_state=0)
-    clf = LinearSVC(random_state=0)
-    cv = GridSearchCV(clf, {'C': [0.1, 1.0]}, scoring="f1")
-    cv.fit(X[:180], y[:180])
-    y_pred = cv.predict(X[180:])
-    C = cv.best_estimator_.C
-
-    clf = LinearSVC(random_state=0)
-    cv = GridSearchCV(clf, {'C': [0.1, 1.0]}, score_func=f1_score)
-    with warnings.catch_warnings(record=True):
-        # catch deprecation warning
-        cv.fit(X[:180], y[:180])
-    y_pred_func = cv.predict(X[180:])
-    C_func = cv.best_estimator_.C
-
-    assert_array_equal(y_pred, y_pred_func)
-    assert_equal(C, C_func)
-
-    # test loss where greater is worse
-    def f1_loss(y_true_, y_pred_):
-        return -f1_score(y_true_, y_pred_)
-
-    clf = LinearSVC(random_state=0)
-    cv = GridSearchCV(clf, {'C': [0.1, 1.0]}, loss_func=f1_loss)
-    with warnings.catch_warnings(record=True):
-        # catch deprecation warning
-        cv.fit(X[:180], y[:180])
-    y_pred_loss = cv.predict(X[180:])
-    C_loss = cv.best_estimator_.C
-
-    assert_array_equal(y_pred, y_pred_loss)
-    assert_equal(C, C_loss)
-
-
 def test_grid_search_precomputed_kernel():
-    """Test that grid search works when the input features are given in the
-    form of a precomputed kernel matrix """
+    # Test that grid search works when the input features are given in the
+    # form of a precomputed kernel matrix
     X_, y_ = make_classification(n_samples=200, n_features=100, random_state=0)
 
     # compute the training kernel matrix corresponding to the linear kernel
@@ -420,8 +387,8 @@ def test_grid_search_precomputed_kernel():
 
 
 def test_grid_search_precomputed_kernel_error_nonsquare():
-    """Test that grid search returns an error with a non-square precomputed
-    training kernel matrix"""
+    # Test that grid search returns an error with a non-square precomputed
+    # training kernel matrix
     K_train = np.zeros((10, 20))
     y_train = np.ones((10, ))
     clf = SVC(kernel='precomputed')
@@ -430,7 +397,7 @@ def test_grid_search_precomputed_kernel_error_nonsquare():
 
 
 def test_grid_search_precomputed_kernel_error_kernel_function():
-    """Test that grid search returns an error when using a kernel_function"""
+    # Test that grid search returns an error when using a kernel_function
     X_, y_ = make_classification(n_samples=200, n_features=100, random_state=0)
     kernel_function = lambda x1, x2: np.dot(x1, x2.T)
     clf = SVC(kernel=kernel_function)
@@ -453,11 +420,9 @@ class BrokenClassifier(BaseEstimator):
 
 
 def test_refit():
-    """Regression test for bug in refitting
-
-    Simulates re-fitting a broken estimator; this used to break with
-    sparse SVMs.
-    """
+    # Regression test for bug in refitting
+    # Simulates re-fitting a broken estimator; this used to break with
+    # sparse SVMs.
     X = np.arange(100).reshape(10, 10)
     y = np.array([0] * 5 + [1] * 5)
 
@@ -466,18 +431,67 @@ def test_refit():
     clf.fit(X, y)
 
 
+def test_gridsearch_nd():
+    # Pass X as list in GridSearchCV
+    X_4d = np.arange(10 * 5 * 3 * 2).reshape(10, 5, 3, 2)
+    y_3d = np.arange(10 * 7 * 11).reshape(10, 7, 11)
+    check_X = lambda x: x.shape[1:] == (5, 3, 2)
+    check_y = lambda x: x.shape[1:] == (7, 11)
+    clf = CheckingClassifier(check_X=check_X, check_y=check_y)
+    grid_search = GridSearchCV(clf, {'foo_param': [1, 2, 3]})
+    grid_search.fit(X_4d, y_3d).score(X, y)
+    assert_true(hasattr(grid_search, "grid_scores_"))
+
+
 def test_X_as_list():
-    """Pass X as list in GridSearchCV"""
+    # Pass X as list in GridSearchCV
     X = np.arange(100).reshape(10, 10)
     y = np.array([0] * 5 + [1] * 5)
 
-    clf = MockListClassifier()
+    clf = CheckingClassifier(check_X=lambda x: isinstance(x, list))
     cv = KFold(n=len(X), n_folds=3)
     grid_search = GridSearchCV(clf, {'foo_param': [1, 2, 3]}, cv=cv)
     grid_search.fit(X.tolist(), y).score(X, y)
     assert_true(hasattr(grid_search, "grid_scores_"))
 
 
+def test_y_as_list():
+    # Pass y as list in GridSearchCV
+    X = np.arange(100).reshape(10, 10)
+    y = np.array([0] * 5 + [1] * 5)
+
+    clf = CheckingClassifier(check_y=lambda x: isinstance(x, list))
+    cv = KFold(n=len(X), n_folds=3)
+    grid_search = GridSearchCV(clf, {'foo_param': [1, 2, 3]}, cv=cv)
+    grid_search.fit(X, y.tolist()).score(X, y)
+    assert_true(hasattr(grid_search, "grid_scores_"))
+
+
+def test_pandas_input():
+    # check cross_val_score doesn't destroy pandas dataframe
+    types = [(MockDataFrame, MockDataFrame)]
+    try:
+        from pandas import Series, DataFrame
+        types.append((DataFrame, Series))
+    except ImportError:
+        pass
+
+    X = np.arange(100).reshape(10, 10)
+    y = np.array([0] * 5 + [1] * 5)
+
+    for InputFeatureType, TargetType in types:
+        # X dataframe, y series
+        X_df, y_ser = InputFeatureType(X), TargetType(y)
+        check_df = lambda x: isinstance(x, InputFeatureType)
+        check_series = lambda x: isinstance(x, TargetType)
+        clf = CheckingClassifier(check_X=check_df, check_y=check_series)
+
+        grid_search = GridSearchCV(clf, {'foo_param': [1, 2, 3]})
+        grid_search.fit(X_df, y_ser).score(X_df, y_ser)
+        grid_search.predict(X_df)
+        assert_true(hasattr(grid_search, "grid_scores_"))
+
+
 def test_unsupervised_grid_search():
     # test grid-search with unsupervised estimator
     X, y = make_blobs(random_state=0)
@@ -494,20 +508,25 @@ def test_unsupervised_grid_search():
     assert_equal(grid_search.best_params_["n_clusters"], 4)
 
 
-def test_bad_estimator():
-    # test grid-search with clustering algorithm which doesn't support
-    # "predict"
-    sc = SpectralClustering()
-    grid_search = GridSearchCV(sc, param_grid=dict(gamma=[.1, 1, 10]),
-                               scoring='ari')
-    assert_raise_message(TypeError, "'score' or a 'predict'", grid_search.fit,
-                         [[1]])
+def test_gridsearch_no_predict():
+    # test grid-search with an estimator without predict.
+    # slight duplication of a test from KDE
+    def custom_scoring(estimator, X):
+        return 42 if estimator.bandwidth == .1 else 0
+    X, _ = make_blobs(cluster_std=.1, random_state=1,
+                      centers=[[0, 1], [1, 0], [0, 0]])
+    search = GridSearchCV(KernelDensity(),
+                          param_grid=dict(bandwidth=[.01, .1, 1]),
+                          scoring=custom_scoring)
+    search.fit(X)
+    assert_equal(search.best_params_['bandwidth'], .1)
+    assert_equal(search.best_score_, 42)
 
 
 def test_param_sampler():
     # test basic properties of param sampler
     param_distributions = {"kernel": ["rbf", "linear"],
-                           "C": distributions.uniform(0, 1)}
+                           "C": uniform(0, 1)}
     sampler = ParameterSampler(param_distributions=param_distributions,
                                n_iter=10, random_state=0)
     samples = [x for x in sampler]
@@ -526,8 +545,8 @@ def test_randomized_search_grid_scores():
     # XXX: as of today (scipy 0.12) it's not possible to set the random seed
     # of scipy.stats distributions: the assertions in this test should thus
     # not depend on the randomization
-    params = dict(C=distributions.expon(scale=10),
-                  gamma=distributions.expon(scale=0.1))
+    params = dict(C=expon(scale=10),
+                  gamma=expon(scale=0.1))
     n_cv_iter = 3
     n_search_iter = 30
     search = RandomizedSearchCV(SVC(), n_iter=n_search_iter, cv=n_cv_iter,
@@ -585,20 +604,20 @@ def test_grid_search_score_consistency():
 
 
 def test_pickle():
-    """Test that a fit search can be pickled"""
+    # Test that a fit search can be pickled
     clf = MockClassifier()
     grid_search = GridSearchCV(clf, {'foo_param': [1, 2, 3]}, refit=True)
     grid_search.fit(X, y)
     pickle.dumps(grid_search)  # smoke test
 
     random_search = RandomizedSearchCV(clf, {'foo_param': [1, 2, 3]},
-                                       refit=True)
+                                       refit=True, n_iter=3)
     random_search.fit(X, y)
     pickle.dumps(random_search)  # smoke test
 
 
 def test_grid_search_with_multioutput_data():
-    """ Test search with multi-output estimator"""
+    # Test search with multi-output estimator
 
     X, y = make_multilabel_classification(return_indicator=True,
                                           random_state=0)
@@ -624,20 +643,8 @@ def test_grid_search_with_multioutput_data():
 
     # Test with a randomized search
     for est in estimators:
-        random_search = RandomizedSearchCV(est, est_parameters, cv=cv)
-        random_search.fit(X, y)
-        for parameters, _, cv_validation_scores in random_search.grid_scores_:
-            est.set_params(**parameters)
-
-            for i, (train, test) in enumerate(cv):
-                est.fit(X[train], y[train])
-                correct_score = est.score(X[test], y[test])
-                assert_almost_equal(correct_score,
-                                    cv_validation_scores[i])
-
-    # Test with a randomized search
-    for est in estimators:
-        random_search = RandomizedSearchCV(est, est_parameters, cv=cv)
+        random_search = RandomizedSearchCV(est, est_parameters,
+                                           cv=cv, n_iter=3)
         random_search.fit(X, y)
         for parameters, _, cv_validation_scores in random_search.grid_scores_:
             est.set_params(**parameters)
@@ -650,7 +657,7 @@ def test_grid_search_with_multioutput_data():
 
 
 def test_predict_proba_disabled():
-    """Test predict_proba when disabled on estimator."""
+    # Test predict_proba when disabled on estimator.
     X = np.arange(20).reshape(5, -1)
     y = [0, 0, 1, 1, 1]
     clf = SVC(probability=False)
@@ -659,7 +666,7 @@ def test_predict_proba_disabled():
 
 
 def test_grid_search_allows_nans():
-    """ Test GridSearchCV with Imputer """
+    # Test GridSearchCV with Imputer
     X = np.arange(20, dtype=np.float64).reshape(5, -1)
     X[2, :] = np.nan
     y = [0, 0, 1, 1, 1]
@@ -668,3 +675,96 @@ def test_grid_search_allows_nans():
         ('classifier', MockClassifier()),
     ])
     GridSearchCV(p, {'classifier__foo_param': [1, 2, 3]}, cv=2).fit(X, y)
+
+
+class FailingClassifier(BaseEstimator):
+    """Classifier that raises a ValueError on fit()"""
+
+    FAILING_PARAMETER = 2
+
+    def __init__(self, parameter=None):
+        self.parameter = parameter
+
+    def fit(self, X, y=None):
+        if self.parameter == FailingClassifier.FAILING_PARAMETER:
+            raise ValueError("Failing classifier failed as required")
+
+    def predict(self, X):
+        return np.zeros(X.shape[0])
+
+
+def test_grid_search_failing_classifier():
+    # GridSearchCV with on_error != 'raise'
+    # Ensures that a warning is raised and score reset where appropriate.
+
+    X, y = make_classification(n_samples=20, n_features=10, random_state=0)
+
+    clf = FailingClassifier()
+
+    # refit=False because we only want to check that errors caused by fits
+    # to individual folds will be caught and warnings raised instead. If
+    # refit was done, then an exception would be raised on refit and not
+    # caught by grid_search (expected behavior), and this would cause an
+    # error in this test.
+    gs = GridSearchCV(clf, [{'parameter': [0, 1, 2]}], scoring='accuracy',
+                      refit=False, error_score=0.0)
+
+    assert_warns(FitFailedWarning, gs.fit, X, y)
+
+    # Ensure that grid scores were set to zero as required for those fits
+    # that are expected to fail.
+    assert all(np.all(this_point.cv_validation_scores == 0.0)
+               for this_point in gs.grid_scores_
+               if this_point.parameters['parameter'] ==
+               FailingClassifier.FAILING_PARAMETER)
+
+    gs = GridSearchCV(clf, [{'parameter': [0, 1, 2]}], scoring='accuracy',
+                      refit=False, error_score=float('nan'))
+    assert_warns(FitFailedWarning, gs.fit, X, y)
+    assert all(np.all(np.isnan(this_point.cv_validation_scores))
+               for this_point in gs.grid_scores_
+               if this_point.parameters['parameter'] ==
+               FailingClassifier.FAILING_PARAMETER)
+
+
+def test_grid_search_failing_classifier_raise():
+    # GridSearchCV with on_error == 'raise' raises the error
+
+    X, y = make_classification(n_samples=20, n_features=10, random_state=0)
+
+    clf = FailingClassifier()
+
+    # refit=False because we want to test the behaviour of the grid search part
+    gs = GridSearchCV(clf, [{'parameter': [0, 1, 2]}], scoring='accuracy',
+                      refit=False, error_score='raise')
+
+    # FailingClassifier issues a ValueError so this is what we look for.
+    assert_raises(ValueError, gs.fit, X, y)
+
+
+def test_parameters_sampler_replacement():
+    # raise error if n_iter too large
+    params = {'first': [0, 1], 'second': ['a', 'b', 'c']}
+    sampler = ParameterSampler(params, n_iter=7)
+    assert_raises(ValueError, list, sampler)
+    # degenerates to GridSearchCV if n_iter the same as grid_size
+    sampler = ParameterSampler(params, n_iter=6)
+    samples = list(sampler)
+    assert_equal(len(samples), 6)
+    for values in ParameterGrid(params):
+        assert_true(values in samples)
+
+    # test sampling without replacement in a large grid
+    params = {'a': range(10), 'b': range(10), 'c': range(10)}
+    sampler = ParameterSampler(params, n_iter=99, random_state=42)
+    samples = list(sampler)
+    assert_equal(len(samples), 99)
+    hashable_samples = ["a%db%dc%d" % (p['a'], p['b'], p['c'])
+                        for p in samples]
+    assert_equal(len(set(hashable_samples)), 99)
+
+    # doesn't go into infinite loops
+    params_distribution = {'first': bernoulli(.5), 'second': ['a', 'b', 'c']}
+    sampler = ParameterSampler(params_distribution, n_iter=7)
+    samples = list(sampler)
+    assert_equal(len(samples), 7)
