@@ -5,7 +5,8 @@ import scipy.sparse as sp
 from scipy.linalg import pinv2
 
 from sklearn.utils.testing import (assert_equal, assert_raises, assert_true,
-                                   assert_almost_equal, assert_array_equal)
+                                   assert_almost_equal, assert_array_equal,
+                                   SkipTest)
 
 from sklearn.utils import check_random_state
 from sklearn.utils import deprecated
@@ -13,11 +14,13 @@ from sklearn.utils import resample
 from sklearn.utils import safe_mask
 from sklearn.utils import column_or_1d
 from sklearn.utils import safe_indexing
+from sklearn.utils import shuffle
 from sklearn.utils.extmath import pinvh
+from sklearn.utils.mocking import MockDataFrame
 
 
 def test_make_rng():
-    """Check the check_random_state utility function behavior"""
+    # Check the check_random_state utility function behavior
     assert_true(check_random_state(None) is np.random.mtrand._rand)
     assert_true(check_random_state(np.random) is np.random.mtrand._rand)
 
@@ -34,12 +37,12 @@ def test_make_rng():
 
 
 def test_resample_noarg():
-    """Border case not worth mentioning in doctests"""
+    # Border case not worth mentioning in doctests
     assert_true(resample() is None)
 
 
 def test_deprecated():
-    """Test whether the deprecated decorator issues appropriate warnings"""
+    # Test whether the deprecated decorator issues appropriate warnings
     # Copied almost verbatim from http://docs.python.org/library/warnings.html
 
     # First a function...
@@ -76,7 +79,7 @@ def test_deprecated():
 
 
 def test_resample_value_errors():
-    """Check that invalid arguments yield ValueError"""
+    # Check that invalid arguments yield ValueError
     assert_raises(ValueError, resample, [0], [0, 1])
     assert_raises(ValueError, resample, [0, 1], [0, 1], n_samples=3)
     assert_raises(ValueError, resample, [0, 1], [0, 1], meaning_of_life=42)
@@ -151,3 +154,35 @@ def test_safe_indexing():
     X_arrays = safe_indexing(np.array(X), inds)
     assert_array_equal(np.array(X_inds), X_arrays)
     assert_array_equal(np.array(X_inds), np.array(X)[inds])
+
+
+def test_safe_indexing_pandas():
+    try:
+        import pandas as pd
+    except ImportError:
+        raise SkipTest("Pandas not found")
+    X = np.array([[1, 2, 3], [4, 5, 6], [7, 8, 9]])
+    X_df = pd.DataFrame(X)
+    inds = np.array([1, 2])
+    X_df_indexed = safe_indexing(X_df, inds)
+    X_indexed = safe_indexing(X_df, inds)
+    assert_array_equal(np.array(X_df_indexed), X_indexed)
+
+
+def test_safe_indexing_mock_pandas():
+    X = np.array([[1, 2, 3], [4, 5, 6], [7, 8, 9]])
+    X_df = MockDataFrame(X)
+    inds = np.array([1, 2])
+    X_df_indexed = safe_indexing(X_df, inds)
+    X_indexed = safe_indexing(X_df, inds)
+    assert_array_equal(np.array(X_df_indexed), X_indexed)
+
+
+def test_shuffle_on_ndim_equals_three():
+    def to_tuple(A):    # to make the inner arrays hashable
+        return tuple(tuple(tuple(C) for C in B) for B in A)
+
+    A = np.array([[[1, 2], [3, 4]], [[5, 6], [7, 8]]])  # A.shape = (2,2,2)
+    S = set(to_tuple(A))
+    shuffle(A)  # shouldn't raise a ValueError for dim = 3
+    assert_equal(set(to_tuple(A)), S)
